
/*************************************************************************************************
This file is part of Simple List Class Sample Project

Copyright (C) 2009 Ezequiel Gastn Miravalles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************************************/

/*************************************************************************************************
Software: Simple List Class Sample Project
Author: Ezequiel Miravalles
Website: http://www.neoegm.com/tech/programming/c-cpp/simple-list-class/
License: GNU GPL v3 (attached, read above)
Last modification: 2009-08-18
*************************************************************************************************/

#include <iostream>
	using std::cout;
	using std::cin;
	using std::endl;

#include "List.h"
#include "String.h"

int main(int argc, char *argv[])
{
	cout << "List sample project" << endl;
	cout << "-------------------" << endl << endl;


	//-----------------------------
	cout << "> Creating a list of strings... An empty string finishes the list..." << endl << endl;

	List<String> lStrings;
	String sTmp;
	
	do
	{
		cout << "> String " << lStrings.Length()+1 << ": ";
		cin >> sTmp;
		
		if (sTmp.Length())
			lStrings.Add(sTmp);
	} while (sTmp.Length());
	
	cout << endl << "> Entry finished. " << lStrings.Length() << " string(s) loaded." << endl << endl;
	
	cout << "> Printing list..." << endl << endl;
	
	cout << lStrings;

	cout << endl;
	//-----------------------------
	
	
	//-----------------------------
	do
	{
		cout << "> Type a string to find in the list (exact match) [empty = end]: ";
		cin >> sTmp;

		if (sTmp.Length())
		{
			int nFound = lStrings.Find(sTmp);
			
			if (nFound != -1)
			{
				lStrings.Delete(nFound);

				cout << "> String \"" << sTmp << "\" found at position " << nFound+1 << " and removed." << endl << endl;
				
				cout << "> Printing list..." << endl << endl;

				if (!lStrings.Empty())
					cout << lStrings;
				else
				{
					cout << "[Empty list]" << endl << endl;
					break;
				}
			}
			else
				cout << "> String \"" << sTmp << "\" not found." << endl;
		}
		
		cout << endl;
	} while (sTmp.Length());
	
	cout << endl;
	//-----------------------------

	//-----------------------------
	cout << "> Now getting a bit more complex... Let's create a list of lists of strings... An empty string finishes the list and an empty first string finishes the list of lists..." << endl << endl;

	List< List<String> > lLists;
	List<String> lTmpList;
	int nStrings = 0;

	do
	{
		lTmpList.DeleteAll();
		
		cout << "> Loading list " << lLists.Length()+1 << "..." << endl;
		
		do
		{
			cout << "\t> String " << lTmpList.Length()+1 << ": ";
			cin >> sTmp;
			
			if (sTmp.Length())
			{
				lTmpList.Add(sTmp);
				nStrings++;
			}
		} while (sTmp.Length());
		
		if (lTmpList.Length())
			lLists.Add(lTmpList);
	} while (lTmpList.Length());
	
	cout << endl << "> Entry finished. " << lLists.Length() << " list(s) loaded, " << nStrings << " string(s) loaded." << endl << endl << endl;

	cout << "> Printing lists (standard method)..." << endl << endl;

	cout << lLists;

	cout << endl;
	
	cout << "> Printing lists (custom method)..." << endl << endl;
	
	for (int i = 0; i < lLists.Length(); i++)
	{
		List<String> *plList = lLists.Elem(i);

		if (plList)
		{
			cout << "- List " << i << endl << endl;
			cout << *plList;
			cout << endl;
		}
	}
	//-----------------------------

    return 0;
}
